@@ -797,7 +797,7 @@ sub decompose {
 		push( @chords, $chord );
 	    }
 	    else {
-		push( @chords, $self->chord($memchords->[$memcrdinx]->chord_display(0)));
+		push( @chords, $self->chord($memchords->[$memcrdinx]->chord_display));
 		warn("Chord recall $in_context\[$memcrdinx]: ", $chords[-1], "\n")
 		  if $config->{debug}->{chords};
 	    }
@@ -2019,7 +2019,7 @@ sub parse_chord {
     # When called from {define} ignore xc/xp.
     $xc = $xp = '' if $def;
 
-    $info = ChordPro::Chords::known_chord( $chord, $def );
+    $info = ChordPro::Chords::known_chord($chord);
     if ( $info ) {
 	warn( "Parsing chord: \"$chord\" found \"",
 	      $info->name, "\" in ", $info->{_via}, "\n" ) if $debug > 1;
@@ -2062,7 +2062,7 @@ sub parse_chord {
 
     if ( $info ) { # TODO roman?
 	# Look it up now, the name may change by transcode.
-	if ( my $i = ChordPro::Chords::known_chord($info, $def) ) {
+	if ( my $i = ChordPro::Chords::known_chord($info) ) {
 	    warn( "Parsing chord: \"$chord\" found ",
 		  $i->name, " for ", $info->name,
 		  " in ", $i->{_via}, "\n" ) if $debug > 1;
@@ -2106,7 +2106,7 @@ sub parse_chord {
     # else: warning has been given.
 
     if ( ! $info ) {
-	if ( my $i = ChordPro::Chords::known_chord($chord, $def) ) {
+	if ( my $i = ChordPro::Chords::known_chord($chord) ) {
 	    $info = $i;
 	    warn( "Parsing chord: \"$chord\" found \"",
 		  $chord, "\" in ",
