@@ -4,9 +4,9 @@
 
 # Author          : Johan Vromans
 # Created On      : Fri Jul  9 14:32:34 2010
-# Last Modified By: Johan Vromans
-# Last Modified On: Sat Feb  5 20:35:50 2022
-# Update Count    : 264
+# Last Modified By: 
+# Last Modified On: Thu Jun 29 09:19:42 2023
+# Update Count    : 269
 # Status          : Unknown, Use with caution!
 
 ################ Common stuff ################
@@ -22,7 +22,7 @@ songs plus chord information. B<chordpro> will then generate a
 photo-ready, professional looking, impress-your-friends sheet-music
 suitable for printing on your nearest printer.
 
-B<chordpro> is a wrapper around L<App::Music::ChordPro>, which does all
+B<chordpro> is a wrapper around module L<ChordPro>, which does all
 of the work.
 
 When invoked as B<a2crd>, or with first argument B<--a2crd>, the input
@@ -37,9 +37,9 @@ use warnings;
 use FindBin;
 use lib "$FindBin::Bin/../CPAN";
 use lib "$FindBin::Bin/../lib";
-use App::Packager qw( :name App::Music::ChordPro );
-use App::Music::ChordPro;
-use App::Music::ChordPro::Utils qw(is_msw);
+use App::Packager qw( :name ChordPro );
+use ChordPro;
+use ChordPro::Utils qw(is_msw);
 
 $ENV{PATH} = join( is_msw() ? ";" : ":",
 		   "$FindBin::Bin", "$FindBin::Bin/..",
